-module(sp). % Sudoku profiling
-author('hanak@emt.bme.hu').
-vsn('2020-11-20').
-export([do/0,do/1,do/2]).

%% See: https://erlang.org/doc/efficiency_guide/users_guide.html
%%      http://erlang.org/doc/apps/tools/cprof_chapter.html
%%      http://erlang.org/doc/apps/tools/fprof_chapter.html

%% WARNING: execution of 'do(fp, ...)' may yield huge files (GBs),
%% and its analysis will consume much time when the sudoku puzzle
%% runs longer than 1 sec. Use do(cp ...) first.

%% Explanation of fprof's analysis output:
%% The CNT column shows the total number of function calls that was found in the trace.
%% In the ACC column is the total time of the trace from first timestamp to last.
%% In the OWN column is the sum of the execution time in functions found in the trace,
%% not including called functions. All time values in the printout are in milliseconds.

tc(Id) ->
    Ids = string:right(integer_to_list(Id), 3 , $0),
    {"tests/test"++Ids++"d.txt", "test"++Ids++"t.txt", "test"++Ids}.

do(all) -> % perform all testcases
    ksudoku:teljes_teszt(5). % parameter: time limit in secs

do(tc, Id) -> % run selected testcase
    ksudoku:stopper(element(1,tc(Id)), element(2,tc(Id)))
    ;
do(cp, Id) -> % run cprof profiling tool on selected testcase
    cprof:stop(),
    cprof:start(),
    R = ksudoku:stopper(element(1,tc(Id)), element(2,tc(Id))),
    cprof:pause(),
    R,
    A = cprof:analyze(sudoku),
    io:format("~p~n", [A])
    ;
do(fp, Id) -> % run fprof profiling tool on selected testcase
    ksudoku:stopper(fprof, element(1,tc(Id)), element(2,tc(Id)), ""),
    file:rename("sudoku.trace", element(3,tc(Id))++".trace"),
    fprof:profile(file, element(3,tc(Id))++".trace"),
    Afile = element(3,tc(Id))++".analysis",
    A = fprof:analyse([{dest, Afile}, {cols, 100}]),
    io:format("~p~nSee the result of the analysis in the file "++Afile++".~n", [A]).

do() ->
    io:format("Usage of Sudoku profiling (sp.erl):~n"),
    io:format("'do(all).'  : perform all testcases found in subfolder 'tests/'.~n"),
    io:format("'do(tc, <n>)': run testcase n (n>0, integer) from subfolder 'tests/'. ~n"),
    io:format("'do(cp, <n>)': run testcase n and perform *cprof* analysis.~n"),
    io:format("'do(fp, <n>)': run testcase n and perform *fprof* analysis.~n~n"),
    io:format("sudoku.beam, ksudoku.beam and sp.beam must exist in the current folder.~n"),
    io:format("  ksudoku.erl must be the new version supplied with this file (sp.erl).~n"),
    io:format("  Do not forget to compile ksudoku.erl, sudoku.erl and sp.erl!~n~n"),
    io:format("Fprof profiling may consume huge disk space and run slowly if the~n"),
    io:format("  Sudoku puzzle needs more than 1 sec to solve.~n"),
    io:format("  Therefore, always make cprof profiling first with a given testcase.~n").
